/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 *
 * This file is part of "andyLib".
 *
 * "andyLib" is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * "andyLib" is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * "andyLib".  If not, see <http://www.gnu.org/licenses/>.
 *
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/andyLib.shtm
 */

package it.andynaz.log;

/**
 * Levels for logs (and configurations).
 * 
 * <p>Each level has a numeric value; the log is written if the log level is
 * high enough (compared with the configuration level).</p>
 * 
 * <p>Available levels (in ascending order) are:
 * <ul>
 *   <li>FINEST</li>
 *   <li>FINE</li>
 *   <li>DEBUG</li>
 *   <li>INFO</li>
 *   <li>WARNING</li>
 *   <li>SEVERE</li>
 * </ul>
 * </p>
 * 
 * @author andynaz
 * @version 2013/01/14
 */
public enum Level{
	
	/**
	 * Lower level.
	 * 
	 * A configuration with this level let you log every event.
	 */
	FINEST(0, "f"),
	
	/**
	 * Low level
	 */
	FINE(2, "F"),
	
	/**
	 * Debug level.
	 *
	 * Used to check for problems during development.
	 */
	DEBUG(4, "D"),
	
	/** 
	 * Defaut level.
	 */
	INFO(6, "I"),
	
	/**
	 * Warning level.
	 *
	 * Should be user for capterued exceptions.
	 */
	WARNING(8, "W"),
	
	/**
	 * Highest level.
	 *
	 * Used for serious problems (like an exit condition for the program).
	 */
	SEVERE(10, "S");
	

	/**
	 * Numeric value of the level.
	 */
	private int level;

	/**
	 * Level code (used in logging).
	 */
	private String code;
	
	
	Level(int level, String code){
		this.level = level;
		this.code = code;
	}

	/**
	 * Returns the code of this level.
	 *
	 * @return the code of this level
	 */
	public String getCode(){
		return this.code;
	}
	
	/**
	 * Test if a log has to e written.
	 * 
	 * A log is written is the log level is higher than the configuration level.
	 * 
	 * <p>This method should be invoked on the configuration level, having like
	 * argument the log level.</p>
	 * 
	 * @param logLevel event level
	 * @return <tt>true</tt> if the log has to e logger, <tt>false</tt> otherwise
	 */
	public boolean hasToLog(Level logLevel){
		return this.level<=logLevel.level;
	}
}
