/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 *
 * This file is part of "andyLib".
 *
 * "andyLib" is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * "andyLib" is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * "andyLib".  If not, see <http://www.gnu.org/licenses/>.
 *
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/andyLib.shtm
 */

package it.andynaz.log;

import static java.lang.System.out;
import it.andynaz.utils.Utils;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Base logger.
 * 
 * This class is the root of all the logger jerarchy. A logger actually logs all
 * the events.
 *
 * <p>Each logger has a set of attributes related to haw it logs the events (for
 * instance, {@link FileLogger} has a File object to where write the logs).<br/>
 * A logger should have also methods related to this set of attributes.<br/>
 * In addition to this members, each logger has also a name, used in the log
 * prefix (useful to distinguish which logger do a log).</p>
 * 
 * <p>Every log is written with a prefix, which is returned by the {@link
 * #getPrefix()} method.</p>
 * 
 * <p>Logs are written using the various <tt>log()</tt> methods: every method,
 * before writing the log, check if the log level if enough to write the log.</p>
 *
 * <p>Sub-classes of this logger can change the way a log is written (for
 * instance, in a file or on a database), only changing the {@link
 * #log(String, Level)} method. For other reasons, like efficiency, aother
 * methods can be overridden.</p>
 * 
 * <p>This logger uses the <tt>System.out</tt></p>.
 * 
 * @author andynaz
 * @version 2013/01/14
 */
public class Logger{
	
	/**
	 * Name of the logger.
	 * 
	 * If present, it is used in the prefix of the log.
	 */
	protected String name;

	/**
	 * Indicates if the date has to be used in the prefix of the log.
	 *
	 * Dafaule value is 'false'.
	 *
	 * <p>If the value is 'true', the prefix starts with "date+separator"
	 */
	protected boolean withDate = false;

	/**
	 * Indicates if the level code has to be used in the profix of the log.
	 *
	 * Dafaule value is 'false'.
	 */
	protected boolean withLevel = false;

	/**
	 * Indicates the level of the logging.
	 *
	 * Dafaule value is {@link Level#INFO}.
	 *
	 * <p>Only the logs with a level higher than this value will be logged.</p>
	 */
	protected Level level = Level.INFO;

	/**
	 * Indicates the format of the date.
	 *
	 * The format is the one used by {@link SimpleDateFormat}. Dafaule value is
	 * "yy-MM-dd HH:mm:ss".
	 */
	protected SimpleDateFormat dateFormat = new SimpleDateFormat("yy-MM-dd HH:mm:ss");

	/**
	 * Indicates the separator to use in the prefix of the logs.
	 *
	 * Dafaule value is "::".
	 */
	protected String separator = "::";

	
	/**
	 * Creates a Logger with a name and a specified configuration.
	 *
	 * From the configuration are taken only the propertyes used. For the
	 * properties not found in the configuration, default values are used.
	 *
	 * <p>Properties are used to construct the attributes of the logger. If error
	 * occour (for instance, the format for the date is not correct), no
	 * exception is thrown and the default value is used.</p>
	 *
	 * <p>The properties taken from the Config object are:
	 * <dl>
	 *   <dt>dateFormat</dt> <dd>format for the date (suitable for a {@link SimpleDateFormat} object</dd>
	 *   <dt>level</dt>      <dd>log level: has to be one of the {@link Level} enum constants</dd>
	 *   <dt>withDate</dt>   <dd>a boolean to indicates if the date has to be used in the log prefix</dd>
	 *   <dt>withLevel</dt>  <dd>a boolean to indicates if the level code has to be used in the log prefix</dd>
	 *   <dt>separator</dt>  <dd>a string indicating the separator of the variuos section of the log prefix</dd>
	 * <dl>
	 * </p>
	 * 
	 * @param name name of the logger
	 * @param conf logger configuration
	 */
	public Logger(String name, Config conf){
		if( !Utils.isEmpty(name) )
			this.name = name;
		
		if(conf==null) return;
		
		// --- formato data
		if( conf.getProperty("dateFormat")!=null ){
			try{
				this.dateFormat = new SimpleDateFormat(conf.getProperty("dateFormat"));
			} catch(Exception e){ /*out.println("err: data"); throw e;*/ }
		}

		// --- livello
		if( conf.getProperty("level")!=null ){
			try{
				this.level = Level.valueOf(conf.getProperty("level"));
			} catch(Exception e){ /*out.println("err: livello"); throw e;*/ }
		}

		// --- log con data
		if( conf.getProperty("withDate")!=null ){
			this.withDate = Boolean.parseBoolean(conf.getProperty("withDate"));
		}

		// --- log con sigla livello
		if( conf.getProperty("withLevel")!=null ){
			this.withLevel = Boolean.parseBoolean(conf.getProperty("withLevel"));
		}

		// --- formato data
		if( conf.getProperty("separator")!=null ){
			this.separator = conf.getProperty("separator");
		}
		
	}
	
	/**
	 * Creates a logger with a name.
	 *
	 * It uses the shared configuration.
	 * 
	 * @param name name of the logger
	 */
	public Logger(String name){
		this(name, Config.getConfig());
	}
	
	/**
	 * Creates a logger with a configuration.
	 * 
	 * @param conf configuration of the logger
	 */
	public Logger(Config conf){
		this(null, conf);
	}
	
	/**
	 * Creates a logger.
	 *
	 * It uses the shared configuration.
	 */
	public Logger(){
		this(null, Config.getConfig());
	}
	
	
	/**
	 * Writes a log with a specified level.
	 * 
	 * Depending on the log configuration, this method include the level code and
	 * a separator before the log.
	 * 
	 * @param log log to be written
	 * @param level log level
	 */
	public void log(String log, Level level){
		if( this.level.hasToLog(level) ){
			System.out.println(getPrefix() + (withLevel?level.getCode()+this.separator:"") + log);
		}
	}

	/**
	 * Writes a log.
	 * 
	 * The level used is {@link Level#INFO}.
	 *
	 * @param log log to be written
	 */
	public void log(String log){
		log(log, Level.INFO);
	}
	
	/**
	 * Writes the log of an exception.
	 *
	 * The level used is {@link Level#INFO}.
	 *
	 * @param e exception to be logged
	 */
	public void log(Exception e){
		log(e, Level.INFO);
	}
	
	/**
	 * Writes the log of an exception with a specificied level.
	 * 
	 * <p>It is written the exception type (with the message of the exception, if
	 * present) and the stack trace</p>
	 *
	 * @param e exception to be logged
	 * @param level log level
	 */
	public void log(Exception e, Level level){
		log(e.toString(), level);
		for( StackTraceElement ste : e.getStackTrace() )
			log("    "+ste.toString(), level);
		Throwable t = e.getCause();
		if( t!=null && t instanceof Exception ){
			log("  caused by:", level);
			log((Exception)t, level);
		}
	}

	
	/**
	 * Returns the prefix to be written before the log.
	 *
	 * <p>The prefix is
	 * <pre>[date+sep][name+sep]</pre>
	 * where each part is optional (depends on the logger configuration).</p>
	 * 
	 * <p>Note: the log prefix returned does not include the level code, which
	 * has to be included in the <tt>log</tt> methods.
	 * 
	 * @return the prefix, based on the configuration of the logger
	 */
	protected String getPrefix(){
		String logPrefix = "";
		if( !Utils.isEmpty(this.name) )
			logPrefix = this.name + this.separator + logPrefix;
		if( this.withDate )
			logPrefix = this.getDate() + this.separator + logPrefix;
		return logPrefix;
	}

	/**
	 * Returns a string with the date.
	 * 
	 * The format of the sctring is the one specified by the {@link #dateFormat}
	 * values.
	 *
	 * @return a string with the date
	 */
	protected String getDate(){
		return this.dateFormat.format(new Date());
	}
	
}
